'use strict';
/* globals Property, Logger, raisedEvent, checkAndStoreIntParameter */

var LOGFILE_NAME = 'event-push-notification.log';
var lInfo = new Logger(LOGFILE_NAME);

var VENTILATION_GROUPS = [ 10, 12, 64 ];
var DEPRECATED_BOOST = 36;
var NEW_BOOST = 6;

function migrateNode (node) {
    var group = node.getChild('group');
    if (!group || VENTILATION_GROUPS.indexOf(group.getValue()) === -1) {
        return true;
    }
    var scene = node.getChild('scene');
    if (!scene || scene.getValue() !== DEPRECATED_BOOST) {
        return true;
    }
    lInfo.logln('--- migrating node: ' + node.getPath());
    return checkAndStoreIntParameter(node.getPath() + '/scene', NEW_BOOST);
}

function migrateTriggerOrAction (node) {    // both have the same structure
    var type = node.getChild('type');
    if (!type || type.getValue() !== 'zone-scene') {
        return true;
    }
    return migrateNode(node);
}

function migrateEntry (entryNode) {
    var i;
    var path = entryNode.getPath();    // value without trailing /
    var success = true;
    var triggers = Property.getNode(path + '/triggers');
    if (triggers) {
        triggers = triggers.getChildren();
        if (triggers && triggers.length) {
            for (i = 0; i < triggers.length; i++) {
                success = success && migrateTriggerOrAction(triggers[i]);
            }
        }
    }
    var conditions = Property.getNode(path + '/conditions/zone-states');
    if (conditions) {
        conditions = conditions.getChildren();
        if (conditions && conditions.length) {
            for (i = 0; i < conditions.length; i++) {
                success = success && migrateNode(conditions[i]);
            }
        }
    }
    return success;
}

function migrateEntries () {
    var root = Property.getNode('values');
    if (!root) {
        lInfo.logln('no root node');
        return true;
    }
    var entries = root.getChildren();
    if (!entries || !entries.length) {
        lInfo.logln('no entries');
        return true;
    }
    lInfo.logln(entries.length + ' entries to check');
    var success = true;
    for (var i = 0; i < entries.length; i++) {
        success = success && migrateEntry(entries[i]);
    }
    return success;
}

if (raisedEvent.name === 'running') {

    Property.load();
    lInfo.logln('MIGRATION of ventilation scene 36 -> 6 START');
    var success = true;

    // 1. check if migration was done before
    var isMigrated = Property.getNode('/scripts/event-push-notification/migratedVentilation36To6');
    if (isMigrated && isMigrated.getValue() === true) {
        lInfo.logln('already migrated');
    } else {
        // 2. migrate
        lInfo.logln('will now migrate...');
        success = migrateEntries();
        lInfo.logln('... success: ' + success);

        // 3. set flag
        Property.setSavedProperty('/scripts/event-push-notification/migratedVentilation36To6', success);
        Property.store();

        // 4. trigger re-creation of all triggers - will happen in pushNotification.js
    }
    lInfo.logln('MIGRATION of ventilation scene 36 -> 6 END');

}
